#
# Copyright (c) Microsoft Corporation.  All rights reserved.
#
import os
import sys
from pathlib import Path
from ..utils.RxHadoopBaseUtils import rx_get_mrs_hadoop_dir_path


def rx_pyspark_check_pyspark_is_loaded():
    return "pyspark" in sys.modules


def rx_pyspark_get_existing_pyspark_context():
    # Not to move import to header. pyspark is optional module
    from pyspark import SparkContext
    return SparkContext._active_spark_context


def rx_pyspark_get_existing_pyspark_session():
    # Not to move import to header. pyspark is optional module
    from pyspark.sql import SparkSession
    return SparkSession.builder.getOrCreate()


def rx_pyspark_check_pyspark_sc_master_type(existing_sc):
    if existing_sc is not None:
        if "yarn" in existing_sc.master:
            return
        elif "spark" in existing_sc.master:
            return
        elif "local" in existing_sc.master:
            return
        else:
            raise Exception("revoscalepy-pyspark-interop does not support %s mode. " % str(existing_sc.master) +
                            "We can only support yarn, standalone and local mode. " +
                            "Please set SparkConf master to 'yarn' when creating SparkContext.")


def rx_pyspark_check_data_is_spark_data_frame(data):
    from pyspark.sql import DataFrame
    return isinstance(data, DataFrame)


def rx_pyspark_check_data_spark_context_still_alive(spark_data_frame):
    return spark_data_frame._sc._jsc is not None


def get_revoscalepy_spark_jar_path():
    hadoop_dir = Path(rx_get_mrs_hadoop_dir_path())
    if hadoop_dir.exists():
        jar_path = Path(os.path.join(str(hadoop_dir), "jar", "scaler-spark_2.11-0.1.0.jar"))
        if jar_path.exists():
            return str(jar_path)

    # if fail to find from mrs-hadoop, then try to use relative path to find it
    current_script_path = Path(os.path.abspath(__file__))
    parent_stack = current_script_path.parents
    # if the parent level size cannot match expectation,
    # then that means mmls is not installed with the correct folder structure
    if len(parent_stack) < 4:
        raise Exception("revoscalepy was installed to incorrect path (%s). " % str(current_script_path) +
                        "Please check your Microsoft Machine Learning Server installation.")
    libraries_dir = parent_stack[3]
    jar_path = libraries_dir.joinpath("common", "hadoop", "jar", "scaler-spark_2.11-0.1.0.jar")
    if not jar_path.exists():
        raise Exception("Cannot locate revoscalepy jar in path (%s). " % str(jar_path) +
                        "Please check your Microsoft Machine Learning Server installation.")
    return str(jar_path)

